@@ -1,6 +1,5 @@
 import requests
 import json
-import httplib
 import os
 import unicodecsv
 import io
@@ -10,11 +9,13 @@ import shutil
 
 class History(object):
 
+    OK = 200
     _BASE_URL = 'https://data.thinknum.com'
 
-    def __init__(self, client_id, client_secret):
+    def __init__(self, client_id, client_secret, proxies={}):
         self._client_id = client_id
         self._client_secret = client_secret
+        self._proxies = proxies
 
         self._token = None
         self._authenticate()
@@ -26,17 +27,30 @@ class History(object):
         }
 
         self._dataset_id = None
+    
+    def _requests(self, method, url, headers={}, data={}, json={}, params={}, stream=False):
+        if method not in ['post', 'get']:
+            raise Exception('Not allowed method')
+        return getattr(requests, method)(
+            url,
+            headers=headers,
+            data=data,
+            json=json,
+            params=params,
+            proxies=self._proxies
+        )
 
     def _authenticate(self):
-        response = requests.post(
-            '{base_url}/api/authorize'.format(base_url=self._BASE_URL), 
+        response = self._requests(
+            method='post',
+            url='{base_url}/api/authorize'.format(base_url=self._BASE_URL), 
             data={
                 "version": "20151130",
                 "client_id": self._client_id,
                 "client_secret": self._client_secret
             }
         )
-        if response.status_code != httplib.OK:
+        if response.status_code != self.OK:
             raise Exception('Invalid autentication')
         self._token = json.loads(response.text)['auth_token']
 
@@ -44,8 +58,9 @@ class History(object):
         if not dataset_id:
             raise Exception('Invalid dataset_id')
 
-        response = requests.get(
-            '{base_url}/datasets/{dataset_id}/history/'.format(
+        response = self._requests(
+            method='get',
+            url='{base_url}/datasets/{dataset_id}/history/'.format(
                 base_url=self._BASE_URL,
                 dataset_id=dataset_id
             ),
@@ -61,8 +76,9 @@ class History(object):
         if not history_date:
             raise Exception('Invalid history_date')
 
-        response = requests.get(
-            '{base_url}/datasets/{dataset_id}/history/{history_date}'.format(
+        response = self._requests(
+            method='get',
+            url='{base_url}/datasets/{dataset_id}/history/{history_date}'.format(
                 base_url=self._BASE_URL,
                 dataset_id=dataset_id,
                 history_date=history_date
@@ -81,8 +97,9 @@ class History(object):
         if not download_path:
             download_path = os.path.abspath(os.getcwd())
 
-        with requests.get(
-            '{base_url}/datasets/{dataset_id}/history/{history_date}'.format(
+        with self._requests(
+            method='get', 
+            url='{base_url}/datasets/{dataset_id}/history/{history_date}'.format(
                 base_url=self._BASE_URL,
                 dataset_id=dataset_id,
                 history_date=history_date
