@@ -1,17 +1,21 @@
 import requests
 import json
-import httplib
 import time
 import sys
 
 
 class Query(object):
 
+    OK = 200
+    CREATED = 201
+    ACCEPTED = 202
+
     _BASE_URL = 'https://data.thinknum.com'
 
-    def __init__(self, client_id, client_secret):
+    def __init__(self, client_id, client_secret, proxies={}):
         self._client_id = client_id
         self._client_secret = client_secret
+        self._proxies = proxies
 
         self._token = None
         self._authenticate()
@@ -31,17 +35,30 @@ class Query(object):
         self._sorts = []
         self._start = 1
         self._limit = 100000
+    
+    def _requests(self, method, url, headers={}, data={}, json={}, params={}):
+        if method not in ['post', 'get', 'head']:
+            raise Exception('Not allowed method')
+        return getattr(requests, method)(
+            url,
+            headers=headers,
+            data=data,
+            json=json,
+            params=params,
+            proxies=self._proxies
+        )
 
     def _authenticate(self):
-        response = requests.post(
-            '{base_url}/api/authorize'.format(base_url=self._BASE_URL), 
+        response = self._requests(
+            method='post',
+            url='{base_url}/api/authorize'.format(base_url=self._BASE_URL),
             data={
                 "version": "20151130",
                 "client_id": self._client_id,
                 "client_secret": self._client_secret
             }
         )
-        if response.status_code != httplib.OK:
+        if response.status_code != self.OK:
             message = response.text
             raise Exception(message)
         self._token = json.loads(response.text)['auth_token']
@@ -124,11 +141,12 @@ class Query(object):
 
     # Return all dataset list
     def get_dataset_list(self):
-        response = requests.get(
-            '{base_url}/datasets/'.format(base_url=self._BASE_URL), 
+        response = self._requests(
+            method='get',
+            url='{base_url}/datasets/'.format(base_url=self._BASE_URL),
             headers=self._headers
         )
-        if response.status_code != httplib.OK:
+        if response.status_code != self.OK:
             message = json.loads(response.text).get("message")
             raise Exception(message)
         return json.loads(response.text).get('datasets', [])
@@ -138,14 +156,15 @@ class Query(object):
         if not dataset_id:
             return {}
 
-        response = requests.get(
-            '{base_url}/datasets/{dataset_id}'.format(
+        response = self._requests(
+            method='get',
+            url='{base_url}/datasets/{dataset_id}'.format(
                 base_url=self._BASE_URL,
                 dataset_id=dataset_id
             ),
             headers=self._headers
         )
-        if response.status_code != httplib.OK:
+        if response.status_code != self.OK:
             message = json.loads(response.text).get("message")
             raise Exception(message)
 
@@ -164,9 +183,9 @@ class Query(object):
     def get_ticker_dataset_list(self, query):
         if not query:
             return []
-
-        response = requests.get(
-            '{base_url}/datasets/'.format(
+        response = self._requests(
+            method='get',
+            url='{base_url}/datasets/'.format(
                 base_url=self._BASE_URL,
                 query=query
             ),
@@ -175,7 +194,7 @@ class Query(object):
                 "ticker": query
             }
         )
-        if response.status_code != httplib.OK:
+        if response.status_code != self.OK:
             message = json.loads(response.text).get("message")
             raise Exception(message)
         return json.loads(response.text).get('datasets', [])
@@ -195,8 +214,13 @@ class Query(object):
                 base_url=self._BASE_URL,
             )
 
-        response = requests.get(url, headers=self._headers, params={"query": query})
-        if response.status_code != httplib.OK:
+        response = self._requests(
+            method='get',
+            url=url,
+            headers=self._headers, 
+            params={"query": query}
+        )
+        if response.status_code != self.OK:
             message = json.loads(response.text).get("message")
             raise Exception(message)
 
@@ -208,25 +232,26 @@ class Query(object):
             return {}
 
         # 1. POST
-        response = requests.post(
-            '{base_url}/datasets/{dataset_id}/query/'.format(
+        response = self._requests(
+            method='post',
+            url='{base_url}/datasets/{dataset_id}/query/'.format(
                 base_url=self._BASE_URL,
                 dataset_id=dataset_id
             ),
             headers=self._headers,
             json=self._query
         )
-        if response.status_code != httplib.CREATED:
+        if response.status_code != self.CREATED:
             message = json.loads(response.text).get("message")
             raise Exception(message)
-
         result = json.loads(response.text)
         query_id = result.get('id')
 
         # 2. HEAD
         while True:
-            response = requests.head(
-                '{base_url}/datasets/{dataset_id}/query/{query_id}'.format(
+            response = self._requests(
+                method='head',
+                url='{base_url}/datasets/{dataset_id}/query/{query_id}'.format(
                     base_url=self._BASE_URL,
                     dataset_id=dataset_id,
                     query_id=query_id
@@ -236,9 +261,9 @@ class Query(object):
                     **{"Accept": "application/vnd.thinknum.table+json"}
                 )
             )
-            if response.status_code == httplib.OK:
+            if response.status_code == self.OK:
                 break
-            elif response.status_code == httplib.ACCEPTED:
+            elif response.status_code == self.ACCEPTED:
                 time.sleep(1)
                 continue
             else:
@@ -254,8 +279,9 @@ class Query(object):
         if self._limit < 1:
             raise Exception('Limit offset is less than 1')
 
-        response = requests.get(
-            '{base_url}/datasets/{dataset_id}/query/{query_id}'.format(
+        response = self._requests(
+            method='get',
+            url='{base_url}/datasets/{dataset_id}/query/{query_id}'.format(
                 base_url=self._BASE_URL,
                 dataset_id=dataset_id,
                 query_id=query_id,
@@ -269,7 +295,7 @@ class Query(object):
                 "limit": self._limit
             }
         )
-        if response.status_code != httplib.OK:
+        if response.status_code != self.OK:
             message = json.loads(response.text).get("message")
             raise Exception(message)
         return json.loads(response.text)
